/*
 * fslurp, Copyright (c) 2012 David Stone <ds.fslurp@asif.com>
 *
 * $RCSfile: fronius.h,v $
 * $Revision: 1.4 $
 * $Date: 2012/10/15 15:44:37 $
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both the
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 */

#ifndef _FRONIUS_H_
#define _FRONIUS_H_

#define FRONIUS_NUM_START_BYTES			3
#define FRONIUS_START_VALUE			((unsigned char)0x80)

struct FroniusRequest
{
	unsigned char startBytes[FRONIUS_NUM_START_BYTES];
	unsigned char length;
	unsigned char deviceOption;
	unsigned char deviceNumber;
	unsigned char command;
	unsigned char checksum;
};

struct FroniusResponseHeader
{
	unsigned char startBytes[FRONIUS_NUM_START_BYTES];
	unsigned char length;
	unsigned char deviceOption;
	unsigned char deviceNumber;
	unsigned char command;
};

// Commands
#define FRONIUS_CMD_GET_VERSION				0x01
#define FRONIUS_CMD_GET_DEVICE_TYPE			0x02
#define FRONIUS_CMD_GET_DATE_TIME			0x03
#define FRONIUS_CMD_GET_ACTIVE_INVERTER_NUMBERS		0x04
#define FRONIUS_CMD_GET_ACTIVE_SENSOR_CARDS		0x05
#define FRONIUS_CMD_GET_SOLAR_NET_STATUS		0x06
#define FRONIUS_CMD_GET_POWER_NOW			0x10
#define FRONIUS_CMD_GET_ENERGY_TOTAL			0x11
#define FRONIUS_CMD_GET_ENERGY_DAY			0x12
#define FRONIUS_CMD_GET_ENERGY_YEAR			0x13
#define FRONIUS_CMD_GET_AC_CURRENT_NOW			0x14
#define FRONIUS_CMD_GET_AC_VOLTAGE_NOW			0x15
#define FRONIUS_CMD_GET_AC_FREQUENCY_NOW		0x16
#define FRONIUS_CMD_GET_DC_CURRENT_NOW			0x17
#define FRONIUS_CMD_GET_DC_VOLTAGE_NOW			0x18
#define FRONIUS_CMD_GET_YIELD_DAY			0x19
#define FRONIUS_CMD_GET_MAXIMUM_POWER_DAY		0x1a
#define FRONIUS_CMD_GET_MAXIMUM_AC_VOLTAGE_DAY		0x1b
#define FRONIUS_CMD_GET_MINIMUM_AC_VOLTAGE_DAY		0x1c
#define FRONIUS_CMD_GET_MAXIMUM_DC_VOLTAGE_DAY		0x1d
#define FRONIUS_CMD_GET_OPERATING_HOURS_DAY		0x1e
#define FRONIUS_CMD_GET_YIELD_YEAR			0x1f
#define FRONIUS_CMD_GET_MAXIMUM_POWER_YEAR		0x20
#define FRONIUS_CMD_GET_MAXIMUM_AC_VOLTAGE_YEAR		0x21
#define FRONIUS_CMD_GET_MINIMUM_AC_VOLTAGE_YEAR		0x22
#define FRONIUS_CMD_GET_MAXIMUM_DC_VOLTAGE_YEAR		0x23
#define FRONIUS_CMD_GET_OPERATING_HOURS_YEAR		0x24
#define FRONIUS_CMD_GET_YIELD_TOTAL			0x25
#define FRONIUS_CMD_GET_MAXIMUM_POWER_TOTAL		0x26
#define FRONIUS_CMD_GET_MAXIMUM_AC_VOLTAGE_TOTAL	0x27
#define FRONIUS_CMD_GET_MINIMUM_AC_VOLTAGE_TOTAL	0x28
#define FRONIUS_CMD_GET_MAXIMUM_DC_VOLTAGE_TOTAL	0x29
#define FRONIUS_CMD_GET_OPERATING_HOURS_TOTAL		0x2a

#define FRONIUS_CMD_GET_TEMPERATURE_CH1_NOW		0xe0
#define FRONIUS_CMD_GET_TEMPERATURE_CH2_NOW		0xe1
#define FRONIUS_CMD_GET_IRRADIANCE_NOW			0xe2
#define FRONIUS_CMD_GET_MINIMUM_TEMPERATURE_CH1_DAY	0xe3
#define FRONIUS_CMD_GET_MAXIMUM_TEMPERATURE_CH1_DAY	0xe4
#define FRONIUS_CMD_GET_MINIMUM_TEMPERATURE_CH1_YEAR	0xe5
#define FRONIUS_CMD_GET_MAXIMUM_TEMPERATURE_CH1_YEAR	0xe6
#define FRONIUS_CMD_GET_MINIMUM_TEMPERATURE_CH1_TOTAL	0xe7
#define FRONIUS_CMD_GET_MAXIMUM_TEMPERATURE_CH1_TOTAL	0xe8
#define FRONIUS_CMD_GET_MINIMUM_TEMPERATURE_CH2_DAY	0xe9
#define FRONIUS_CMD_GET_MAXIMUM_TEMPERATURE_CH2_DAY	0xea
#define FRONIUS_CMD_GET_MINIMUM_TEMPERATURE_CH2_YEAR	0xeb
#define FRONIUS_CMD_GET_MAXIMUM_TEMPERATURE_CH2_YEAR	0xec
#define FRONIUS_CMD_GET_MINIMUM_TEMPERATURE_CH2_TOTAL	0xed
#define FRONIUS_CMD_GET_MAXIMUM_TEMPERATURE_CH2_TOTAL	0xee
#define FRONIUS_CMD_GET_MAXIMUM_IRRADIANCE_DAY		0xef
#define FRONIUS_CMD_GET_MAXIMUM_IRRADIANCE_YEAR		0xf0
#define FRONIUS_CMD_GET_MAXIMUM_IRRADIANCE_TOTAL	0xf1
#define FRONIUS_CMD_GET_DIGITAL_CH1_NOW			0xf2
#define FRONIUS_CMD_GET_DIGITAL_CH2_NOW			0xf3
#define FRONIUS_CMD_GET_MAXIMUM_DIGITAL_CH1_DAY		0xf4
#define FRONIUS_CMD_GET_MAXIMUM_DIGITAL_CH1_YEAR	0xf5
#define FRONIUS_CMD_GET_MAXIMUM_DIGITAL_CH1_TOTAL	0xf6
#define FRONIUS_CMD_GET_MAXIMUM_DIGITAL_CH2_DAY		0xf7
#define FRONIUS_CMD_GET_MAXIMUM_DIGITAL_CH2_YEAR	0xf8
#define FRONIUS_CMD_GET_MAXIMUM_DIGITAL_CH2_TOTAL	0xf9

#define FRONIUS_ERROR					0x0e

// Protocol Error Info

#define FRONIUS_PROTOCOL_ERROR_UNKNOWN_COMMAND		0x01
#define FRONIUS_PROTOCOL_ERROR_TIMEOUT			0x02
#define FRONIUS_PROTOCOL_ERROR_INCORRECT_DATA_STRUCTURE	0x03
#define FRONIUS_PROTOCOL_ERROR_COMMAND_QUEUE_FULL	0x04
#define FRONIUS_PROTOCOL_ERROR_DEVICE_OR_OPTION_NA	0x05
#define FRONIUS_PROTOCOL_ERROR_NO_RESPONSE		0x06
#define FRONIUS_PROTOCOL_ERROR_SENSOR_ERROR		0x07
#define FRONIUS_PROTOCOL_ERROR_SENSOR_NOT_ACTIVE	0x08
#define FRONIUS_PROTOCOL_ERROR_INCORRECT_COMMAND	0x09
#define FRONIUS_PROTOCOL_ERROR_DEVICE_ID_COLLISION	0x0a

// values for the device/option byte
#define FRONIUS_INTERFACE_CARD			0
#define FRONIUS_INVERTER			1
#define FRONIUS_SENSOR_CARD			2
#define FRONIUS_IG_DATA_LOGGER			3
#define FRONIUS_OPTION_ID_RESERVED		4
#define FRONIUS_STRING_CONTROL			5

// Values for Inteface types
#define FRONIUS_INTERFACE_RS_232		1
#define FRONIUS_INTERFACE_CARD_EASY		2
#define FRONIUS_INTERFACE_RS_422		3

// values for the Solar Net Status
#define FRONIUS_SOLAR_NET_OK			1
#define FRONIUS_SOLAR_NET_ERROR			0

#define FRONIUS_MAXIMUM_INVERTERS		100

#endif /* _FRONIUS_H_ */

